package com.jeeplus.business.finance.service;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.jeeplus.business.finance.domain.*;
import com.jeeplus.business.finance.mapper.*;
import com.jeeplus.business.finance.service.dto.JyFinanceInvoiceBaseDTO;
import com.jeeplus.business.finance.service.dto.JyFinanceInvoiceDTO;
import com.jeeplus.business.finance.service.dto.JyFinanceInvoiceDetailDTO;
import com.jeeplus.business.finance.service.dto.JyFinanceInvoiceReceivablesDTO;
import com.jeeplus.business.finance.service.mapstruct.*;
import com.jeeplus.business.project.mapper.JyProjectMapper;
import com.jeeplus.business.workClientInfo.domain.dto.JyWorkClientInfosDto;
import com.jeeplus.business.workClientInfo.service.JyWorkClientService;
import com.jeeplus.common.TokenProvider;
import com.jeeplus.core.query.QueryWrapperGenerator;
import com.jeeplus.sys.domain.User;
import com.jeeplus.sys.feign.IUserApi;
import com.jeeplus.sys.feign.IWorkAttachmentApi;
import com.jeeplus.sys.service.dto.UserDTO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;


@Service
@Transactional(rollbackFor = Exception.class)
public class JyFinanceInvoiceService extends ServiceImpl<JyFinanceInvoiceMapper, JyFinanceInvoice> {

    @Resource
    private JyFinanceInvoiceMapper cwFinanceInvoiceMapper;
    @Resource
    private JyFinanceInvoiceBaseMapper cwFinanceInvoiceBaseMapper;
    @Resource
    private JyFinanceInvoiceDetailMapper cwFinanceInvoiceDetailMapper;
    @Resource
    private JyFinanceInvoiceReceivablesMapper cwFinanceInvoiceReceivablesMapper;
    @Resource
    private JyFinanceInvoiceInvalidMapper cwFinanceInvoiceInvalidMapper;
    @Resource
    private JyProjectMapper cwProjectRecordsMapper;



//    @Resource
//    private SerialnumTplService serialnumTplService;
//    @Resource
//    private OssServiceMapper ossServiceMapper;
//    @Resource
//    private UserService userService;
//    @Resource
//    private OssService ossService;
    @Resource
    private JyWorkClientService jyWorkClientService;

    public List<String> getSearchList(ArrayList<String> searchIdList, List<String> ids) {
        List<String> newSearchIdList = searchIdList.stream().filter(item -> {
            AtomicInteger num = new AtomicInteger();
            ids.stream().forEach(id -> {
                if (item.equals(id)) {
                    num.getAndIncrement();
                }
            });
            if (num.get() > 0) {
                return true;
            }
            return false;
        }).collect(Collectors.toList());
        return newSearchIdList;
    }

    public IPage<JyFinanceInvoiceDTO> findList(Page<JyFinanceInvoiceDTO> page, JyFinanceInvoiceDTO jyFinanceInvoiceDTO) throws Exception{
        QueryWrapper<JyFinanceInvoice> queryWrapper = QueryWrapperGenerator.buildQueryCondition ( JyFinanceInvoiceWrapper.INSTANCE.toEntity(jyFinanceInvoiceDTO), JyFinanceInvoice.class );
        queryWrapper.eq("fi.del_flag","0");
        queryWrapper.eq("a.del_flag","0");
        ArrayList<String> searchIdList = new ArrayList<>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        AtomicInteger checkNum = new AtomicInteger();
        if (ObjectUtil.isNotEmpty(jyFinanceInvoiceDTO)) {
            //发票编号查询
            if (StringUtils.isNotBlank(jyFinanceInvoiceDTO.getNo())){
                queryWrapper.like("fi.no",jyFinanceInvoiceDTO.getNo());
            }
            // 发票号查询
            if (StringUtils.isNotBlank(jyFinanceInvoiceDTO.getNumber())) {
                List<JyFinanceInvoiceDetail> cwFinanceInvoiceDetails = cwFinanceInvoiceDetailMapper.selectList(new LambdaQueryWrapper<JyFinanceInvoiceDetail>().like(JyFinanceInvoiceDetail::getNumber, jyFinanceInvoiceDTO.getNumber()));
                List<String> ids = cwFinanceInvoiceDetails.stream().distinct().map(JyFinanceInvoiceDetail::getInvoiceId).collect(Collectors.toList());
                if(CollectionUtil.isEmpty(ids)) {
                    return new Page<JyFinanceInvoiceDTO>();
                }else{
                    if (CollectionUtil.isNotEmpty(searchIdList)){
                        List<String> newSearchIdList = getSearchList(searchIdList,ids);
                        searchIdList = new ArrayList<String>();
                        for (String item : newSearchIdList) {
                            searchIdList.add(item);
                        }
                    }else{
                        if (checkNum.get() > 0){
                            return new Page<JyFinanceInvoiceDTO>();
                        }else{
                            for (String item : ids) {
                                searchIdList.add(item);
                            }
                        }
                    }
                    checkNum.getAndIncrement();
                }
            }
            // 开票总金额
            if (StringUtils.isNotBlank(jyFinanceInvoiceDTO.getAccountBegin())&&StringUtils.isNotBlank(jyFinanceInvoiceDTO.getAccountEnd())) {
                List<JyFinanceInvoice> cwFinanceInvoices = cwFinanceInvoiceMapper.selectList(new LambdaQueryWrapper<JyFinanceInvoice>()
                        .between(JyFinanceInvoice::getAccount, new BigDecimal(jyFinanceInvoiceDTO.getAccountBegin()), new BigDecimal(jyFinanceInvoiceDTO.getAccountEnd())));
                List<String> ids = cwFinanceInvoices.stream().distinct().map(JyFinanceInvoice::getId).collect(Collectors.toList());
                if(CollectionUtil.isEmpty(ids)) {
                    return new Page<JyFinanceInvoiceDTO>();
                }else{
                    if (CollectionUtil.isNotEmpty(searchIdList)){
                        List<String> newSearchIdList = getSearchList(searchIdList,ids);
                        searchIdList = new ArrayList<String>();
                        for (String item : newSearchIdList) {
                            searchIdList.add(item);
                        }
                    }else{
                        if (checkNum.get() > 0){
                            return new Page<JyFinanceInvoiceDTO>();
                        }else{
                            for (String item : ids) {
                                searchIdList.add(item);
                            }
                        }
                    }
                    checkNum.getAndIncrement();
                }
            }
            // 开票日期
            if (StringUtils.isNotBlank(jyFinanceInvoiceDTO.getBillingDateBegin())&&StringUtils.isNotBlank(jyFinanceInvoiceDTO.getBillingDateEnd())) {
                List<JyFinanceInvoice> cwFinanceInvoices = cwFinanceInvoiceMapper.selectList(new LambdaQueryWrapper<JyFinanceInvoice>()
                        .between(JyFinanceInvoice::getBillingDate, sdf.parse(jyFinanceInvoiceDTO.getBillingDateBegin()), sdf.parse(jyFinanceInvoiceDTO.getBillingDateEnd())));
                List<String> ids = cwFinanceInvoices.stream().distinct().map(JyFinanceInvoice::getId).collect(Collectors.toList());
                if(CollectionUtil.isEmpty(ids)) {
                    return new Page<JyFinanceInvoiceDTO>();
                }else{
                    if (CollectionUtil.isNotEmpty(searchIdList)){
                        List<String> newSearchIdList = getSearchList(searchIdList,ids);
                        searchIdList = new ArrayList<String>();
                        for (String item : newSearchIdList) {
                            searchIdList.add(item);
                        }
                    }else{
                        if (checkNum.get() > 0){
                            return new Page<JyFinanceInvoiceDTO>();
                        }else{
                            for (String item : ids) {
                                searchIdList.add(item);
                            }
                        }
                    }
                    checkNum.getAndIncrement();
                }
            }
            // 收款日期
            if (StringUtils.isNotBlank(jyFinanceInvoiceDTO.getRemittanceDateBegin())&&StringUtils.isNotBlank(jyFinanceInvoiceDTO.getRemittanceDateEnd())) {
                List<JyFinanceInvoice> cwFinanceInvoices = cwFinanceInvoiceMapper.selectList(new LambdaQueryWrapper<JyFinanceInvoice>()
                        .between(JyFinanceInvoice::getReceivablesDate, sdf.parse(jyFinanceInvoiceDTO.getRemittanceDateBegin()), sdf.parse(jyFinanceInvoiceDTO.getRemittanceDateEnd())));
                List<String> ids = cwFinanceInvoices.stream().distinct().map(JyFinanceInvoice::getId).collect(Collectors.toList());
                if(CollectionUtil.isEmpty(ids)) {
                    return new Page<JyFinanceInvoiceDTO>();
                }else{
                    if (CollectionUtil.isNotEmpty(searchIdList)){
                        List<String> newSearchIdList = getSearchList(searchIdList,ids);
                        searchIdList = new ArrayList<String>();
                        for (String item : newSearchIdList) {
                            searchIdList.add(item);
                        }
                    }else{
                        if (checkNum.get() > 0){
                            return new Page<JyFinanceInvoiceDTO>();
                        }else{
                            for (String item : ids) {
                                searchIdList.add(item);
                            }
                        }
                    }
                    checkNum.getAndIncrement();
                }
            }
            // 项目名称
            if (StringUtils.isNotBlank(jyFinanceInvoiceDTO.getProgramId())){
                    List<JyFinanceInvoiceBase> cwFinanceInvoiceBaseList = cwFinanceInvoiceBaseMapper.selectList(new LambdaQueryWrapper<JyFinanceInvoiceBase>()
                            .eq(JyFinanceInvoiceBase::getProgramId, jyFinanceInvoiceDTO.getProgramId()));
                    List<String> ids = cwFinanceInvoiceBaseList.stream().distinct().map(JyFinanceInvoiceBase::getInvoiceId).collect(Collectors.toList());
                    if(CollectionUtil.isEmpty(ids)) {
                        return new Page<JyFinanceInvoiceDTO>();
                    }else{
                        if (CollectionUtil.isNotEmpty(searchIdList)){
                            List<String> newSearchIdList = getSearchList(searchIdList,ids);
                            searchIdList = new ArrayList<String>();
                            for (String item : newSearchIdList) {
                                searchIdList.add(item);
                            }
                        }else{
                            if (checkNum.get() > 0){
                                return new Page<JyFinanceInvoiceDTO>();
                            }else{
                                for (String item : ids) {
                                    searchIdList.add(item);
                                }
                            }
                        }
                        checkNum.getAndIncrement();

                    }
            }
            // 经办人
            if (StringUtils.isNotBlank(jyFinanceInvoiceDTO.getOperator())){
                queryWrapper.like("su.name", jyFinanceInvoiceDTO.getOperator());
            }
            //报备类型
            if (StringUtils.isNotBlank(jyFinanceInvoiceDTO.getReportType())){
                queryWrapper.eq("fi.report_type",jyFinanceInvoiceDTO.getReportType()).or(i->i.eq("pr.report_type",jyFinanceInvoiceDTO.getReportType()));
            }

            // 对账人
            if (StringUtils.isNotBlank(jyFinanceInvoiceDTO.getReconciliationPeopleName())) {
                List<User> users = SpringUtil.getBean ( IUserApi.class ).selectListByName(jyFinanceInvoiceDTO.getReconciliationPeopleName());
                List<String> userIdList = users.stream().distinct().map(User::getId).collect(Collectors.toList());
                if (CollectionUtil.isNotEmpty(userIdList)) {
                    queryWrapper.in("fi.reconciliation_people",userIdList);
                } else {
                    return new Page<JyFinanceInvoiceDTO>();
                 }
            }
            // 经办人部门
            if (StringUtils.isNotBlank(jyFinanceInvoiceDTO.getOperatorOffice())){
                List<User> userList = SpringUtil.getBean ( IUserApi.class ).getUserByOfficeAll(jyFinanceInvoiceDTO.getOperatorOffice());
                if (CollectionUtil.isNotEmpty(userList)) {
                    List<String> userIdList = userList.stream().distinct().map(User::getId).collect(Collectors.toList());
                    if (CollectionUtil.isNotEmpty(userIdList)) {
                        queryWrapper.in("fi.create_by_id",userIdList);
                    } else {
                        return new Page<JyFinanceInvoiceDTO>();
                    }
                } else {
                    return new Page<JyFinanceInvoiceDTO>();
                }
            }
        }
        if (CollectionUtil.isNotEmpty(searchIdList)){
            List<String> ids = searchIdList.stream().distinct().collect(Collectors.toList());
            queryWrapper.in("fi.id",ids);
        } else{
            if (checkNum.get() > 0) {
                return new Page<JyFinanceInvoiceDTO>();
            }
        }
        // 如果当前用户是部门主任，则可以查看部门所有的项目、自己创建的项目以及所属项目组的项目
        // 如果当前用户是员工，则可以查看自己创建的项目以及所属项目组的项目
        StringBuilder officeIds = new StringBuilder();
//        UserDTO userDTO = SpringUtil.getBean ( IUserApi.class ).getByToken(TokenProvider.getCurrentToken());
        List<String> manageOfficeIdList= Lists.newArrayList();
        //判定是否为会计所长
//        if(CommonUtils.haveRoleKjsz()){
//            //获取当前人管理的部门id
//            if(StringUtils.isNotBlank(userDTO.getManageOfficeIds())){
//                manageOfficeIdList = Arrays.asList(userDTO.getManageOfficeIds().split(","));
//            }else{
//                manageOfficeIdList.add(userDTO.getOfficeDTO().getId());
//            }
//            for (int i = 0; i <manageOfficeIdList.size(); i++){
//                officeIds.append("'").append(manageOfficeIdList.get(i)).append("'");
//                if(i < manageOfficeIdList.size()-1){
//                    officeIds.append(",");
//                }
//            }
//        }


        //获取当前登录人信息
//        UserDTO userDTO = SpringUtil.getBean ( IUserApi.class ).getByToken(TokenProvider.getCurrentToken ( ));
        //获取当前登录人所属角色
        //若是评估部门主任，则仅展示部门主任信息
        //若是评估员工，则仅展示自己的数据信息
//        List<RoleDTO> roleDTOList = userDTO.getRoleDTOList();
//        for (RoleDTO roleDTO : roleDTOList) {
//            //根据角色id查询角色信息
//            RoleDTO roleDTOById = SpringUtil.getBean(IRoleApi.class).getRoleDTOById(roleDTO.getId());
//            if("cwyg".equals(roleDTOById.getEnName()) && "兴光会计员工".equals(roleDTOById.getName())){
//                queryWrapper.eq("a.create_by_id", userDTO.getId());
//                break;
//            }else if("cw_bmzr".equals(roleDTOById.getEnName()) && "兴光会计部门主任".equals(roleDTOById.getName())){
//                queryWrapper.eq("sumo.user_id", userDTO.getId());
//                break;
//            }else if("szfbdagly".equals(roleDTOById.getEnName()) && "苏州分部档案管理员".equals(roleDTOById.getName())){
//                queryWrapper.eq("d.office_id", "1613800824395530242");
//                break;
//            }
//        }
        IPage<JyFinanceInvoiceDTO> list = cwFinanceInvoiceMapper.findList(page, queryWrapper);

        return list;
    }

    public JyFinanceInvoiceDTO queryById(String id) {

        JyFinanceInvoiceDTO cwFinanceInvoiceDTO = cwFinanceInvoiceMapper.queryById(id);

        // 电话号获取
        if (ObjectUtil.isNotEmpty(cwFinanceInvoiceDTO) && StringUtils.isNotBlank(cwFinanceInvoiceDTO.getBillingId())) {
            JyWorkClientInfosDto cwWorkClientBilling = jyWorkClientService.findById(cwFinanceInvoiceDTO.getBillingId());
            if (ObjectUtil.isNotEmpty(cwWorkClientBilling) && StringUtils.isNotBlank(cwWorkClientBilling.getWorkClientInfo().getTelephone())) {
                // 如果通过billingId(开票信息id)可以查询到数据，并且数据的phone(电话)有值的话，那么就将方法返回结果的telPhone替换为查询到的phone
                cwFinanceInvoiceDTO.setTelPhone(cwWorkClientBilling.getWorkClientInfo().getTelephone());
            } else {
                // 如果通过billingId(开票信息id)查询不到数据或者查询到数据的phone值为空
                // 将billingId(开票信息id)置空，因为前端根据billingId是否有值来判断->是否允许电话号码可以手填
                cwFinanceInvoiceDTO.setBillingId("");
                // 将telPhone(电话号)置空
//                cwFinanceInvoiceDTO.setTelPhone("");
            }
        }

        // 实际开票单位名称获取
        if (ObjectUtil.isNotEmpty(cwFinanceInvoiceDTO) && StringUtils.isNotBlank(cwFinanceInvoiceDTO.getBillingWorkplaceRealId())) {
            JyWorkClientInfosDto cwWorkClientBase = jyWorkClientService.findById(cwFinanceInvoiceDTO.getBillingWorkplaceRealId());
            if (ObjectUtil.isNotEmpty(cwWorkClientBase) && StringUtils.isNotBlank(cwWorkClientBase.getWorkClientInfo().getName())) {
                // 如果通过billingWorkplaceRealId(实际开票单位id)可以查询到数据，并且数据的name(实际开票单位名称)有值的话，
                // 那么就将方法返回结果的billingWorkplaceReal替换
                cwFinanceInvoiceDTO.setBillingWorkplaceReal(cwWorkClientBase.getWorkClientInfo().getName());
            }
        }





        return cwFinanceInvoiceDTO;
    }

    public JyFinanceInvoice saveForm(JyFinanceInvoiceDTO cwFinanceInvoiceDTO) throws Exception{
        JyFinanceInvoice cwFinanceInvoice = JyFinanceInvoiceWrapper.INSTANCE.toEntity(cwFinanceInvoiceDTO);
        if (ObjectUtil.isNotEmpty(cwFinanceInvoice)) {
            if(StringUtils.isBlank(cwFinanceInvoice.getId())){
                //获取当前登录人信息
                UserDTO userDTO = SpringUtil.getBean ( IUserApi.class ).getByToken(TokenProvider.getCurrentToken());
                //发票编号生成
                String serialNum = SpringUtil.getBean ( IWorkAttachmentApi.class ).genSerialNum(userDTO.getCompanyDTO().getId(), JyFinanceInvoiceDTO.BIZ_CODE,TokenProvider.getCurrentToken());
                cwFinanceInvoice.setNo(serialNum);
                if (StringUtils.isBlank(cwFinanceInvoice.getInvalidStatus())){
                    cwFinanceInvoice.setInvalidStatus("0");
                }
                if (StringUtils.isBlank(cwFinanceInvoice.getReceivablesStatus())){
                    cwFinanceInvoice.setReceivablesStatus("0");
                }
                if(StringUtils.isNotEmpty(cwFinanceInvoice.getReportType())){
                    cwFinanceInvoice.setReportType(cwFinanceInvoiceDTO.getReportType());
                }

            }
        }
        if(StringUtils.isNotBlank(cwFinanceInvoiceDTO.getId())){
            cwFinanceInvoiceMapper.updateInfoById(cwFinanceInvoice);
        }else{
            cwFinanceInvoiceMapper.insert(cwFinanceInvoice);
        }

        if (ObjectUtil.isNotEmpty(cwFinanceInvoiceDTO)) {
            // 如果当前是执行的是收款页面的保存方法
            if ("1".equals(cwFinanceInvoiceDTO.getIsReceivables())) {
                // 如果传过来的收款日期为空，则将本条发票的收款日期置空
                if (ObjectUtil.isEmpty(cwFinanceInvoiceDTO.getReceivablesDate())) {
                    // 根据发票id，置空发票的收款日期
                    cwFinanceInvoiceMapper.setReceivablesDateNull(cwFinanceInvoice.getId());
                }
            }
            // 基本信息保存
            if(CollectionUtil.isNotEmpty(cwFinanceInvoiceDTO.getFinanceInvoiceBaseDTOList())){
                List<String> delIds = cwFinanceInvoiceDTO.getFinanceInvoiceBaseDTOList().stream().distinct().filter(item->{
                    if(StringUtils.isNotBlank(item.getId())){
                        return true;
                    }
                    return false;
                }).map(JyFinanceInvoiceBaseDTO::getId).collect(Collectors.toList());
                if(CollectionUtil.isNotEmpty(delIds)){
                    cwFinanceInvoiceBaseMapper.delete(new QueryWrapper<JyFinanceInvoiceBase>().lambda()
                            .eq(JyFinanceInvoiceBase::getInvoiceId, cwFinanceInvoice.getId()).notIn(JyFinanceInvoiceBase::getId,delIds));
                }else{
                    cwFinanceInvoiceBaseMapper.delete(new QueryWrapper<JyFinanceInvoiceBase>().lambda()
                            .eq(JyFinanceInvoiceBase::getInvoiceId, cwFinanceInvoice.getId()));
                }
                cwFinanceInvoiceDTO.getFinanceInvoiceBaseDTOList().stream().forEach(item->{
                    JyFinanceInvoiceBase cwFinanceInvoiceBase = JyFinanceInvoiceBaseWrapper.INSTANCE.toEntity(item);

                    cwFinanceInvoiceBase.setAccount(item.getAccount());
                    if(StringUtils.isNotBlank(cwFinanceInvoiceBase.getId())){
                        cwFinanceInvoiceBaseMapper.updateById(cwFinanceInvoiceBase);
                    }else{
                        cwFinanceInvoiceBase.setInvoiceId(cwFinanceInvoice.getId());
                        cwFinanceInvoiceBaseMapper.insert(cwFinanceInvoiceBase);
                    }
                    if (StringUtils.isNotEmpty(cwFinanceInvoiceBase.getProgramId())){
                        if (null!=cwProjectRecordsMapper.getById(cwFinanceInvoiceBase.getProgramId())){
                            cwFinanceInvoiceBase.setType("1");
                            cwFinanceInvoiceBaseMapper.updateById(cwFinanceInvoiceBase);
                            cwFinanceInvoice.setRecordType(cwFinanceInvoiceBase.getType());
                        }
                    }else {
                        cwFinanceInvoiceBase.setType("0");
                        cwFinanceInvoiceBaseMapper.updateById(cwFinanceInvoiceBase);
                        cwFinanceInvoice.setRecordType(cwFinanceInvoiceBase.getType());
                    }

                });
            }else{
                cwFinanceInvoiceBaseMapper.delete(new QueryWrapper<JyFinanceInvoiceBase>().lambda()
                        .eq(JyFinanceInvoiceBase::getInvoiceId, cwFinanceInvoice.getId()));
            }
            // 附件
            Map<String,String> map = new HashMap<>();
            String fileList = JSON.toJSONString((cwFinanceInvoiceDTO.getWorkAttachmentDtoList()));
            String attachmentId = cwFinanceInvoice.getId();
            String attachmentFlag = "jy_invoice";
            map.put("fileList",fileList);
            map.put("attachmentId",attachmentId);
            map.put("attachmentFlag",attachmentFlag);
            map.put("currentToken", TokenProvider.getCurrentToken ( ));
            SpringUtil.getBean ( IWorkAttachmentApi.class ).saveOrUpdateFileList(map);

            // 发票明细保存
            if(CollectionUtil.isNotEmpty(cwFinanceInvoiceDTO.getFinanceInvoiceDetailDTOList())){
                List<String> delIds = cwFinanceInvoiceDTO.getFinanceInvoiceDetailDTOList().stream().distinct().filter(item -> {
                    if (StringUtils.isNotBlank(item.getId())) {
                        return true;
                    }
                    return false;
                }).map(JyFinanceInvoiceDetailDTO::getId).collect(Collectors.toList());
                if(CollectionUtil.isNotEmpty(delIds)){
                    cwFinanceInvoiceDetailMapper.delete(new QueryWrapper<JyFinanceInvoiceDetail>().lambda()
                            .eq(JyFinanceInvoiceDetail::getInvoiceId, cwFinanceInvoice.getId())
                            .notIn(JyFinanceInvoiceDetail::getId,delIds));
                }else{
                    cwFinanceInvoiceDetailMapper.delete(new QueryWrapper<JyFinanceInvoiceDetail>().lambda().eq(JyFinanceInvoiceDetail::getInvoiceId, cwFinanceInvoice.getId()));
                }
                cwFinanceInvoiceDTO.getFinanceInvoiceDetailDTOList().stream().forEach(item->{
                    JyFinanceInvoiceDetail cwFinanceInvoiceDetail = JyFinanceInvoiceDetailWrapper.INSTANCE.toEntity(item);
                    if(StringUtils.isNotBlank(cwFinanceInvoiceDetail.getId())){
                        cwFinanceInvoiceDetailMapper.updateById(cwFinanceInvoiceDetail);
                    }else{
                        cwFinanceInvoiceDetail.setInvoiceId(cwFinanceInvoice.getId());
                        cwFinanceInvoiceDetailMapper.insert(cwFinanceInvoiceDetail);
                    }
                });
            }else{
                cwFinanceInvoiceDetailMapper.delete(new QueryWrapper<JyFinanceInvoiceDetail>().lambda().eq(JyFinanceInvoiceDetail::getInvoiceId, cwFinanceInvoice.getId()));
            }
            // 收款明细保存
            if(CollectionUtil.isNotEmpty(cwFinanceInvoiceDTO.getFinanceInvoiceReceivablesDTOList())){
                List<String> delIds = cwFinanceInvoiceDTO.getFinanceInvoiceReceivablesDTOList().stream().distinct().filter(item -> {
                    if (StringUtils.isNotBlank(item.getId())) {
                        return true;
                    }
                    return false;
                }).map(JyFinanceInvoiceReceivablesDTO::getId).collect(Collectors.toList());
                if(CollectionUtil.isNotEmpty(delIds)){
                    cwFinanceInvoiceReceivablesMapper.delete(new QueryWrapper<JyFinanceInvoiceReceivables>().lambda()
                            .eq(JyFinanceInvoiceReceivables::getInvoiceId, cwFinanceInvoice.getId())
                            .notIn(JyFinanceInvoiceReceivables::getId,delIds));
                }else{
                    cwFinanceInvoiceReceivablesMapper.delete(new QueryWrapper<JyFinanceInvoiceReceivables>().lambda().eq(JyFinanceInvoiceReceivables::getInvoiceId, cwFinanceInvoice.getId()));
                }
                cwFinanceInvoiceDTO.getFinanceInvoiceReceivablesDTOList().stream().forEach(item->{
                    JyFinanceInvoiceReceivables cwFinanceInvoiceReceivables = JyFinanceInvoiceReceivablesWrapper.INSTANCE.toEntity(item);
                    if(StringUtils.isNotBlank(cwFinanceInvoiceReceivables.getId())){
                        cwFinanceInvoiceReceivablesMapper.updateById(cwFinanceInvoiceReceivables);
                    }else{
                        cwFinanceInvoiceReceivables.setInvoiceId(cwFinanceInvoice.getId());
                        cwFinanceInvoiceReceivablesMapper.insert(cwFinanceInvoiceReceivables);
                    }
                });
            }else{
                cwFinanceInvoiceReceivablesMapper.delete(new QueryWrapper<JyFinanceInvoiceReceivables>().lambda().eq(JyFinanceInvoiceReceivables::getInvoiceId, cwFinanceInvoice.getId()));
            }
            // 作废信息保存
            if(ObjectUtil.isNotEmpty(cwFinanceInvoiceDTO.getFinanceInvoiceInvalidDTO())){
                JyFinanceInvoiceInvalid cwFinanceInvoiceInvalid = JyFinanceInvoiceInvalidWrapper.INSTANCE.toEntity(cwFinanceInvoiceDTO.getFinanceInvoiceInvalidDTO());
                if(StringUtils.isNotBlank(cwFinanceInvoiceInvalid.getId())){
                    cwFinanceInvoiceInvalidMapper.update(cwFinanceInvoiceInvalid,
                            new QueryWrapper<JyFinanceInvoiceInvalid>().lambda().eq(JyFinanceInvoiceInvalid::getId, cwFinanceInvoiceInvalid.getId()));
                }else{
                    cwFinanceInvoiceInvalid.setInvalidStatus("0");
                    cwFinanceInvoiceInvalid.setInvoiceId(cwFinanceInvoice.getId());
                    cwFinanceInvoiceInvalidMapper.insert(cwFinanceInvoiceInvalid);
                }
            }else{
                cwFinanceInvoiceInvalidMapper.delete(new QueryWrapper<JyFinanceInvoiceInvalid>().lambda().eq(JyFinanceInvoiceInvalid::getInvoiceId, cwFinanceInvoice.getId()));
                JyFinanceInvoiceInvalid cwFinanceInvoiceInvalid = new JyFinanceInvoiceInvalid();
                cwFinanceInvoiceInvalid.setInvalidStatus("0");
                cwFinanceInvoiceInvalid.setInvoiceId(cwFinanceInvoice.getId());
                cwFinanceInvoiceInvalidMapper.insert(cwFinanceInvoiceInvalid);
            }
        }
        return cwFinanceInvoice;
    }

    public ResponseEntity saveInvoice(JyFinanceInvoiceDTO cwFinanceInvoiceDTO) throws Exception{
        this.saveForm(cwFinanceInvoiceDTO);
        return ResponseEntity.ok("保存成功");
    }

    public String saveFormInvalid(JyFinanceInvoiceDTO cwFinanceInvoiceDTO) throws Exception{
        String invoiceId = this.saveForm(cwFinanceInvoiceDTO).getId();
        List<JyFinanceInvoiceInvalid> cwFinanceInvoiceInvalids = cwFinanceInvoiceInvalidMapper.selectList(
                new QueryWrapper<JyFinanceInvoiceInvalid>().lambda().eq(JyFinanceInvoiceInvalid::getInvoiceId, invoiceId));
        return cwFinanceInvoiceInvalids.get(0).getId();
    }

    public ResponseEntity deleteByIds(String ids) {
        String idArray[] =ids.split(",");
        this.removeByIds (Lists.newArrayList (idArray));
        return ResponseEntity.ok ("删除成功");
    }

    public Boolean queryByNumber(String number, String id) {
        List<JyFinanceInvoiceDetail> cwFinanceInvoiceDetails = cwFinanceInvoiceDetailMapper.selectList(new QueryWrapper<JyFinanceInvoiceDetail>().lambda()
                .eq(JyFinanceInvoiceDetail::getNumber, number)
                .eq(JyFinanceInvoiceDetail::getDelFlag, '0')
                .ne(StringUtils.isNotBlank(id), JyFinanceInvoiceDetail::getId,id));
        if (CollectionUtil.isNotEmpty(cwFinanceInvoiceDetails)) {
            return true;
        }
        return false;
    }

    public String queryIdByInvalidId(String id) {
        JyFinanceInvoiceInvalid cwFinanceInvoiceInvalid = cwFinanceInvoiceInvalidMapper.selectById(id);
        return cwFinanceInvoiceInvalid.getInvoiceId();
    }

    public String updateStatusById(JyFinanceInvoiceDTO cwFinanceInvoiceDTO) {
        JyFinanceInvoice cwFinanceInvoice = JyFinanceInvoiceWrapper.INSTANCE.toEntity(cwFinanceInvoiceDTO);
        if ("4".equals(cwFinanceInvoice.getStatus())) {
            cwFinanceInvoiceMapper.updateInfoById(cwFinanceInvoice);
        }else{
            cwFinanceInvoiceMapper.update(cwFinanceInvoice, new QueryWrapper<JyFinanceInvoice>().lambda().eq(JyFinanceInvoice::getId, cwFinanceInvoice.getId()));
        }
        return "操作成功";
    }

    public String isReceivables(JyFinanceInvoiceDTO cwFinanceInvoiceDTO) {
        JyFinanceInvoice cwFinanceInvoice = JyFinanceInvoiceWrapper.INSTANCE.toEntity(cwFinanceInvoiceDTO);
        cwFinanceInvoiceMapper.update(cwFinanceInvoice, new QueryWrapper<JyFinanceInvoice>().lambda().eq(JyFinanceInvoice::getId, cwFinanceInvoice.getId()));
        return "操作成功";
    }

    public String importDecide(List<JyFinanceInvoiceDetailDTO> list, ArrayList<JyFinanceInvoiceDetailDTO> arrayList, HashMap<String,String> hashMap){
        for (JyFinanceInvoiceDetailDTO cwFinanceInvoiceDetailDTO : list) {
            if(ObjectUtil.isEmpty(cwFinanceInvoiceDetailDTO)){
                continue;
            }

            if(StringUtils.isBlank(cwFinanceInvoiceDetailDTO.getNumber())){
                return "文件中有发票号为空，请重新填写";
            }
            if(StringUtils.isNotBlank(cwFinanceInvoiceDetailDTO.getNumber())){
                try {
                    Integer integer = new Integer(cwFinanceInvoiceDetailDTO.getNumber());
                    if (integer < 0) {
                        return "文件中有发票号填写不正确，请重新填写";
                    }
                }catch (Exception e){
                    return "文件中有发票号填写不正确，请重新填写";
                }
                List<JyFinanceInvoiceDetail> cwFinanceInvoiceDetailList = cwFinanceInvoiceDetailMapper.selectList(new QueryWrapper<JyFinanceInvoiceDetail>().eq("number", cwFinanceInvoiceDetailDTO.getNumber()));
                if(CollectionUtil.isNotEmpty(cwFinanceInvoiceDetailList)){
                    return "文件中有发票号已在系统中存在，请重新填写";
                }
            }
//            if (StringUtils.isBlank(financeInvoiceDetailDTO.getRate())) {
//                return "文件中有税率为空，请重新填写";
//            }
            if (StringUtils.isNotBlank(cwFinanceInvoiceDetailDTO.getRate())) {
                try {
                    if ((new BigDecimal(cwFinanceInvoiceDetailDTO.getRate())).compareTo(new BigDecimal(1)) == -1
                            || (new BigDecimal(cwFinanceInvoiceDetailDTO.getRate())).compareTo(new BigDecimal(100)) == 1) {
                        return "文件中有税率填写不正确，请重新填写";
                    }
                }catch (Exception e){
                    return "文件中有税率填写不正确，请重新填写";
                }
            }
            if (StringUtils.isBlank(cwFinanceInvoiceDetailDTO.getAccount())) {
                return "文件中有开票金额为空，请重新填写";
            } else {
                try {
                    cwFinanceInvoiceDetailDTO.setAccount(new BigDecimal(cwFinanceInvoiceDetailDTO.getAccount()).toString());
                }catch (Exception e){
                    return "文件中有开票金额填写不正确，请重新填写";
                }
            }

            hashMap.put(cwFinanceInvoiceDetailDTO.getNumber(),null);

            arrayList.add(cwFinanceInvoiceDetailDTO);
        }
        return null;
    }

    public List<JyFinanceInvoiceDTO> getByIds(String projectId,JyFinanceInvoiceDTO cwFinanceInvoiceDTO) throws Exception{
        if(StringUtils.isNotBlank(projectId)){
            List<String> idByProjectId = cwFinanceInvoiceMapper.getIdByProjectId(projectId);
            if (CollectionUtil.isNotEmpty(idByProjectId)){
                QueryWrapper<JyFinanceInvoice> queryWrapper = QueryWrapperGenerator.buildQueryCondition ( JyFinanceInvoiceWrapper.INSTANCE.toEntity(cwFinanceInvoiceDTO), JyFinanceInvoice.class );
                queryWrapper.in("fi.id", idByProjectId);
                queryWrapper.eq("fi.del_flag","0");
                queryWrapper.eq("fi.status","5"); // 发票申请流程已通过
                queryWrapper.orderByDesc("fi.create_time");
                queryWrapper.eq("fi.invalid_status","0"); // 未作废
                return cwFinanceInvoiceMapper.getByIds(queryWrapper);
            }
        }
        return new ArrayList<>();
    }

    public List<JyFinanceInvoiceDTO> getByContractId(String contractId,JyFinanceInvoiceDTO cwFinanceInvoiceDTO) throws Exception{
        if(StringUtils.isNotBlank(contractId)){
            List<String> idByContractId = cwFinanceInvoiceMapper.getIdByContractId(contractId);
            if (CollectionUtil.isNotEmpty(idByContractId)){
                QueryWrapper<JyFinanceInvoice> queryWrapper = QueryWrapperGenerator.buildQueryCondition ( JyFinanceInvoiceWrapper.INSTANCE.toEntity(cwFinanceInvoiceDTO), JyFinanceInvoice.class );
                queryWrapper.in("fi.id", idByContractId);
                queryWrapper.eq("fi.del_flag","0");
                queryWrapper.eq("fi.status","5"); // 发票申请流程已通过
                queryWrapper.orderByDesc("fi.create_time");
                queryWrapper.eq("fi.invalid_status","0"); // 未作废
                return cwFinanceInvoiceMapper.getByIds(queryWrapper);
            }
        }
        return new ArrayList<>();
    }

    public List<JyFinanceInvoiceDTO> getIdByClientId(String clientId,JyFinanceInvoiceDTO cwFinanceInvoiceDTO) throws Exception{
        if(StringUtils.isNotBlank(clientId)){
            List<String> idByClientId = cwFinanceInvoiceMapper.getIdByClientId(clientId);
            if (CollectionUtil.isNotEmpty(idByClientId)){
                QueryWrapper<JyFinanceInvoice> queryWrapper = QueryWrapperGenerator.buildQueryCondition ( JyFinanceInvoiceWrapper.INSTANCE.toEntity(cwFinanceInvoiceDTO), JyFinanceInvoice.class );
                queryWrapper.in("fi.id", idByClientId);
                queryWrapper.eq("fi.del_flag","0");
                queryWrapper.eq("fi.status","5"); // 发票申请流程已通过
                queryWrapper.orderByDesc("fi.create_time");
                queryWrapper.eq("fi.invalid_status","0"); // 未作废
                return cwFinanceInvoiceMapper.getByIds(queryWrapper);
            }
        }
        return new ArrayList<>();
    }
}
