package com.jeeplus.pubmodules.reimApprovalType.controller;

import com.jeeplus.pubmodules.reimApprovalType.domain.CwReimbursementTypeInfo;
import com.jeeplus.pubmodules.reimApprovalType.mapper.CwReimbursementTypeMapper;
import com.jeeplus.pubmodules.reimApprovalType.service.CwReimbursementTypeForTreeDataService;
import com.jeeplus.pubmodules.reimApprovalType.service.CwReimbursementTypeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author: 王强
 * @create: 2022-11-25 09:18
 **/
@RestController
@Api(tags ="财务-报销类型管理")
@RequestMapping(value = "/reimbursementApproval/type")
public class CwReimbursementTypeController {

    @Resource
    private CwReimbursementTypeService service;

    @Resource
    private CwReimbursementTypeForTreeDataService treeService;

    @Resource
    private CwReimbursementTypeMapper mapper;

    /**
     * 列表查询
     * @param info
     * @return
     */
    @ApiOperation(value = "列表查询")
    @GetMapping("/list")
    public ResponseEntity<List<CwReimbursementTypeInfo>> list(CwReimbursementTypeInfo info) {
        List<CwReimbursementTypeInfo> list = service.list(info);
        return ResponseEntity.ok(list);
    }

    /**
     * 列表查询
     * @param info
     * @return
     */
    @ApiOperation(value = "列表查询")
    @GetMapping("/bxList")
    public ResponseEntity<List<CwReimbursementTypeInfo>> bxList(CwReimbursementTypeInfo info) {
        List<CwReimbursementTypeInfo> list = service.bxList2(info);
        return ResponseEntity.ok(list);
    }

    /**
     * 采购类型列表查询
     * @param info
     * @return
     */
    @ApiOperation(value = "采购类型列表查询")
    @GetMapping("/cgList")
    public ResponseEntity<List<CwReimbursementTypeInfo>> cgList(CwReimbursementTypeInfo info) {
        List<CwReimbursementTypeInfo> list = service.cgList(info);
        return ResponseEntity.ok(list);
    }

    /**
     * 新增/修改
     * @param info
     * @return
     */
    @ApiOperation(value = "新增/修改")
    @PostMapping("/save")
    public ResponseEntity<String> save(@RequestBody CwReimbursementTypeInfo info) {
        String s = service.save(info);
        return ResponseEntity.ok(s);
    }

    /**
     * 根据id查询
     * @param id
     * @return
     */
    @ApiOperation(value = "根据id查询")
    @GetMapping("/findById")
    public ResponseEntity<CwReimbursementTypeInfo> findById(String id) {
        CwReimbursementTypeInfo info = service.getById(id);
        return ResponseEntity.ok(info);
    }

    /**
     * 根据id删除
     * @param id
     * @return
     */
    @ApiOperation(value = "根据id删除")
    @GetMapping("/deleteById")
    public ResponseEntity<String> deleteById(String id) {
        mapper.deleteById(id);
        return ResponseEntity.ok("操作完成");
    }

    /**
     * 查询树形
     * @param extId 排除的ID
     * @return
     */
    @ApiOperation(value = "查询树形")
    @GetMapping("/treeData")
    public ResponseEntity<List<CwReimbursementTypeInfo>> treeData(@RequestParam(required = false) String extId, @RequestParam(required = false) String type) throws Exception{
        List<CwReimbursementTypeInfo> infos = treeService.treeDataForType(extId, type);
        return ResponseEntity.ok(infos);
    }

    /**
     * 查询树形
     * @param extId 排除的ID
     * @return
     */
    @ApiOperation(value = "查询树形")
    @GetMapping("/treeData1")
    public ResponseEntity<List<CwReimbursementTypeInfo>> treeData1(@RequestParam(required = false) String extId, @RequestParam(required = false) String type) throws Exception{
        List<CwReimbursementTypeInfo> infos = treeService.treeDataForType1(extId, type);
        return ResponseEntity.ok(infos);
    }

    /**
     * 查询树形
     * @param extId 排除的ID
     * @return
     */
    @ApiOperation(value = "查询树形")
    @GetMapping("/summaryTreeData")
    public ResponseEntity<List<CwReimbursementTypeInfo>> summaryTreeData(@RequestParam(required = false) String extId, @RequestParam(required = false) String type) throws Exception{
        List<CwReimbursementTypeInfo> infos = treeService.summaryTreeData(extId, type);
        return ResponseEntity.ok(infos);
    }

    /**
     * 查询树形
     * @param extId 排除的ID
     * @return
     */
    @ApiOperation(value = "查询树形")
    @GetMapping("/treeData2")
    public ResponseEntity<List<CwReimbursementTypeInfo>> treeData2(@RequestParam(required = false) String extId, @RequestParam(required = false) String type) throws Exception{
        List<CwReimbursementTypeInfo> infos = treeService.treeDataForType2(extId, type);
        return ResponseEntity.ok(infos);
    }

}
